#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "data.h"
#include "predistort.h"

#define PIXEL_TIME 500			// pixel length

#define CW_DIT_TIME	1200/10		// 10 WPM

#define QRSS_DIT_TIME	6000		// dit interval
#define FSKCW_SHIFT	100		// about half the hell max shift

#define PAUSE_TIME 10000		// between idents

#define KEY _BV(PB0)
#define FSK _BV(PB1)

enum element { DIT = 1, DAH = 3, CHAR_SPACE = 2, WORD_SPACE = 6, PAUSE = 10 };
enum speed { CW, QRSS };
enum modulation { MOD_OOK, MOD_FSK };

void init() {
	// keyed when low set high
	PORTB = 0;

	// Setup OC0B as outputs in fast PWM mode with no IO clock prescaler
	TCCR0A = _BV(COM0B1) | _BV(WGM01) | _BV(WGM00);
	TCCR0B = _BV(CS00);

	// output setup
	DDRB = KEY | FSK;

}

void key() { PORTB |= KEY; }
void unkey() { PORTB &= (unsigned char)~KEY; }

void shift(char amount) { OCR0B = amount; }

void sendHell() {
	unsigned int i;
	unsigned char j, row;
	for(i = 0; i < HELL_LENGTH; i++) {
		row = pgm_read_byte(&hellData[i]);
		for(j = 0; j < 8; j++) {
			shift(pgm_read_byte(&hellShift[j]));
			if(row & (1 << j)) key();
			_delay_ms(PIXEL_TIME);
			unkey();
		}
	}
}

void delay(int element, int speed) {
	int i;
	for(i = 0; i < element; i++) {
		switch(speed) {
			case CW:	_delay_ms(CW_DIT_TIME);		break;
			case QRSS:	_delay_ms(QRSS_DIT_TIME);	break;
		}
	}
}

void sendCW(int modulation, int speed) {
	int i;
	unsigned char j;

	shift(0);
	if(modulation == MOD_FSK) key();
	delay(PAUSE, speed);

	for(i = 0; i < CW_LENGTH; i++) {
		for(j = 0; j < 4; j++) {
			switch((pgm_read_byte(&cwData[i]) >> (6-(j*2))) & 0x03) {
				case 0:
					delay(WORD_SPACE, speed);
					break;
				case 1:
					delay(CHAR_SPACE, speed);
					break;
				case 2:
					(modulation == MOD_OOK)?( key() ):( shift(FSKCW_SHIFT) );
					delay(DIT, speed);
					(modulation == MOD_OOK)?( unkey() ):( shift(0) );
					delay(DIT, speed);
					break;
				case 3:
					(modulation == MOD_OOK)?( key() ):( shift(FSKCW_SHIFT) );
					delay(DAH, speed);
					(modulation == MOD_OOK)?( unkey() ):( shift(0) );
					delay(DIT, speed);
					break;
			}
		}
	}

	delay(PAUSE, speed);
	if(modulation == MOD_FSK) unkey();
	shift(0);
}

int main() {
	init();

	while(1) {
		sendCW(MOD_OOK, CW);
		_delay_ms(PAUSE_TIME);

		sendHell();
		_delay_ms(PAUSE_TIME);

		sendCW(MOD_FSK, QRSS);
		_delay_ms(PAUSE_TIME);

	}

	return 0; // unreached
}
